function [y, w] = TCExt( X, periods, type, index,  mu, bound, epsilon, maxIter, trace)
% TCExt-- Teomporally correlated signals extraction algorithm based on 2th and 4th statistics.
%         This algorithm can be divided into two parts. The first part uses a 2th order statistics
%         based algorithm to roughly extract the desired signal, and the second part uses a 
%         Newton-lik extraction algorithm based on higher-order statistics to process
%         the noisy desired signal as clean as possible. Note the input signal X
%         should be WHITENED.
%             If type = 1, then the code implements the algorithm in Ref.[1] below; 
%             If type = 2, then the code implements the algorithm in Ref.[3] below.
%        If there are several source signals having the same period length T, then
%        the algorithm still extracts these signals one by one, providing that they
%        have different autocorrelation values at lag T.
%
% Command:
%    [y,w] = TCExt( X, periods, type, index,  mu, bound, epsilon, maxIter, trace)
%    [y,w] = TCExt( X, periods, type, index,  mu, bound, epsilon, maxIter)
%    [y,w] = TCExt( X, periods, type, index,  mu, bound, epsilon)
%    [y,w] = TCExt( X, periods, type, index,  mu, bound)
%    [y,w] = TCExt( X, periods, type, index,  mu)
%    [y,w] = TCExt( X, periods, type, index)
%    [y,w] = TCExt( X, periods, type)
%
% Parameters:
%          y --- extracted source signal, according to the given optimal
%                time delay.
%          w --- extraction weight vector, that is, y=w'*X. 
%          X --- prewhitened observed mixed signals. Each row is a mixed
%                signal.
%    periods --- lags of peaks in the autocorrelation of the desired source signal
%      bound --- the distance between the weight vector in stage two and the one in stage 
%                one should be less than this parameter. Default value: 0.5
%      index --- if there are several source signals having the same period length T,
%                then INDEX indicates which source signal is extracted that
%                has the INDEX largest autocorrelation value at the lag T. Default value: 1
%         mu --- step-size for the one-unit ICA sub-algorithm in Stage Two
%    epsilon --- convergence criterion, say, 0.0001. When the absolute changed value 
%                of weighted vector is less than it, then the algorithm stops. Default value: 0.001             
%    maxIter --- maximum iterations for estimating each independent component. Default value: 500      
%       type --- 1-algorithm based on autocorrelation and standard kurtosis, see Ref.[1] below
%                2-algorithm based on autocorrelation and the extraction algorithm
%                based on maximum likelihood contrast and adaptive nonlinearities, see Ref.[3] below
%                Default value: 2
%      trace --- If trace==1, print out the progress of the algorithm
%                If trace==0, do not print out
%                Default value: 1
%
% See also:
%         TCExtdemo     cICA      wkExt      eigBSE
%
% Reference:
%    [1] Zhi-Lin Zhang, Liqing Zhang: A two-stage based approach to
%        effectively extract periodic signals. Proc. of the 6th International Conference on
%        Independent Component Analysis and Blind Signal Separation (ICA 2006). Lecture Notes 
%        in Computer Science, vol 3889, pp.303-310 
%    [2] Zhi-Lin Zhang, Zhang Yi, Robust  Extraction of Specific Signals with 
%        Temporal Structure. Neurocomputing 69 (7-9) 888-893 
%    [3] Karvanen, J.,Eriksson, J., and Koivunen, V.: "Pearson System Based Method 
%        for Blind Separation", Proceedings of Second International Workshop on Independent 
%        Component Analysis and Blind Signal Separation, Helsinki 2000, pp. 585-590
%
% Author: Zhilin Zhang (zhangzlacademy@gmail.com)
%         http://dsp.ucsd.edu/~zhilin/
%
% version: 1.5.2   Date: May   6, 2006
% version: 1.5.1   Date: May   4, 2006
% version: 1.5     Date: Apr. 20, 2006
% version: 1.4     Date: Apr. 13, 2006
% version: 1.3     Date: Apr.  5, 2006
% version: 1.2     Date: Apr.  4, 2006
% version: 1.1     Date: Apr.  3, 2006
% version: 1.0     Date: Feb.  8, 2006
% 



[ICnum, IClen]=size(X);

Default_bound = 0.5;
Default_index = 1;
Default_epsilon = 0.0001;
Default_maxIter = 200;
Default_type = 2;
Default_trace = 1;
Default_mu = 0.5;


if (nargin < 2),
    error('At least 2 parameters are required');
elseif (nargin == 2),
    type = Default_type;  index = Default_index; mu = Default_mu;  
    bound = Default_bound;  epsilon = Default_epsilon; maxIter = Default_maxIter; 
    trace = Default_trace;
elseif (nargin == 3),
    index = Default_index; mu = Default_mu;  
    bound = Default_bound;  epsilon = Default_epsilon; maxIter = Default_maxIter; 
    trace = Default_trace;    
elseif (nargin == 4)
    mu = Default_mu;  
    bound = Default_bound;  epsilon = Default_epsilon; maxIter = Default_maxIter; 
    trace = Default_trace;    
elseif (nargin == 5)
    bound = Default_bound;  epsilon = Default_epsilon; maxIter = Default_maxIter; 
    trace = Default_trace;    
elseif (nargin == 6)
    epsilon = Default_epsilon; maxIter = Default_maxIter;  trace = Default_trace;   
elseif (nargin == 7)
    maxIter = Default_maxIter;  trace = Default_trace;   
elseif (nargin == 8)
    trace = Default_trace;   
end;




if trace==1
    fprintf('\n=================================================================\n');
    fprintf('                     Running TCExt Algorithm \n');
    fprintf('=================================================================\n');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%% The First Part of the Algorithm: Coarse Extraction %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% calling the function EIGBSE, which extracts the temporally-correlated signal using
% the second-order statistics. After convergence, the obtained weight vector w1 will
% be used as the initial weight vector of the higher-order statistics based algorithm
% in the second part. See Ref.[1,2] for details.
if trace==1
    fprintf('[The First Part of the Algorithm]\n');
    fprintf('Extracting Based on the Second-Order Statistics\n');
    fprintf('Use eigenvalue decomposition. \n');
end

Rx = zeros(size(X,1));
for k = periods
    tempo = X(:,k+1:IClen) * X(:,1: IClen-k)' / (IClen-k);
    Rx = Rx + tempo;
end
clear tempo;
Rx = (Rx + Rx')/2;
[V,D] = eig(Rx);  
[puiss,II] = sort( diag(D) );

eigs = flipud(diag(D));
numSource = find( eigs > eigs(1)*0.1 );
if trace==1 
    fprintf('The first largest %d eigenvalues are :\n',numSource(end));
    disp(eigs(1:numSource(end)));
    fprintf('Note: There may be %d source signals with the same autocorrelation value.\n',numSource(end)); 
    fprintf('      You are extracting the No.%d source signal.\n',index);
end

w1 = V(:,II(ICnum-index+1));  % select the eigenvector corresponding to the INDEX largest eigenvalue
w1 = w1/norm(w1);

if (trace == 1)   fprintf('Roughly obtain the weight vector w1 !\n\n');  end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%% The Second Part of the Algorithm: Fine Extraction %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if trace==1
    fprintf('[The Second Part of the Algorithm]\n');
    fprintf('Extracting Based on the Higher-Order Statistics\n');
end

flag = 1;   % When flag==1,the algorithm continues; when flag==0, the algorithm stops
loop = 1;   % iteration times

if type==1
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Extracting signals using the one-unit fixed-point algorithm based on kurtosis.
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if trace==1  fprintf('The fixed-point algorithm based on kurtosis was used.\n'); end

    while (flag == 1)

        if (loop == 1)
            w = w1;
            oldw = w1;
        end

        % one-unit fixed-point algorithm based on kurtosis
        w =  X * ( X'* w).^3 / IClen  -  3 * w ;
        w = w/norm(w);

        distance = 1-abs(w'*w1);   % change between the current w and w1
        if distance < bound
            % If the current w is near to w1, then judge whether the
            % algorithm has converged.
            wchange = 1-abs(w'*oldw);
            if trace==1   fprintf('No.%d iteration: change in w is %g\n',loop, wchange);  end;
            if wchange < epsilon
                if trace==1   fprintf('Converged after %d iteration\n',loop);   end;
                flag = 0;
            end

        else
            % If the current w is not near to w1, then change current w to w1,
            % added to a small random deviation. This step is important
            % to guarantee that the solution is spatially near w1
            if trace==1
                fprintf('After No.%d iteration, the distance between current weight and w1 is %g\n',loop,distance);
                fprintf('Clearly, the current weight is far from the one derived from the first stage, \n');
                fprintf('therefore we change the current weight to w1, added a small random deviation\n');
            end
            w = w1 + 0.05 * randn(size(w));
            w = w/norm(w);
        end

        if loop >= maxIter
            if trace==1
                fprintf('\n------------------------------------ Notice ------------------------------------\n');
                fprintf('After %d iterations, the algorithm still cannot converge, implying that the weight\n',loop);
                fprintf('vector w1 from the first stage may be more reliable than the one from the second stage.\n');
                fprintf('Thus we use w1 as the final weigtht vector !\n');
                fprintf('--------------------------------------------------------------------------------\n');
            end;
            w = w1;
            flag = 0;
        end

        oldw = w;
        loop = loop + 1;

    end  % end of WHILE loop


elseif type == 2
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Use the extraction algorithm based on maximum likelihood contrast.  The
    % algorithm adopts several adaptive score functions and fixed score functions.
    % (1)If the desired source signal is clearly super-Gaussian, then the adaptive score
    %    functions derived from the t-distribution are used.
    % (2)If the desired source signal is clearly sub-Gaussian, then the cubic
    %    function y(x)=x^3 derived from the generalized Gaussian distribution is
    %    used.
    % (3)If the desired source signal is near-Gaussian, then the adaptive score
    %    functions derived from the generalized Beta distribution (GBD) and the
    %    Pearson system are used.
    % To select the suitable score functions or distribution, we use the estimated
    % 2th,3th,4th moment of the estimated signal.
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



    % set the initial value of the 4th,3th,2th moment
    M4 = 0;  M3 = 0;  M2 = 1;

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Build up the looking-up table of kurtosis-beta relationship for the t-distribution
    Beta = linspace(4,30,2000);
    Ktable = 3*gamma( (Beta-4)./2 ) .* gamma(Beta./2) ./ gamma( (Beta-2)./2 ).^2 - 3;

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % core part of the algorithm
    while (flag == 1)

        if (loop == 1)
            if trace==1
                fprintf('Starting the one-unit extraction algorithm based on maximum likelihood contrast\n');
            end
            w = w1;     % use the initialized weight vector, derived from the first part of the algorithm
            oldw = w1;
        end

        y = w' * X;

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % estimate kurtosis and moments of the desired source signal
        kurt = M4/M2^2 - 3 ;   fgt = 0.1;
        M4 = fgt * M4 + (1-fgt) * moment(y,4);
        M2 = fgt * M2 + (1-fgt) * moment(y,2);
        M3 = fgt * M3 + (1-fgt) * moment(y,3);

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Selection of score functions according to the estimated moments
        if M4 > M3^2 + 4
            % If the desired source signal is strongly super-Gaussian, then
            % use the adaptive score functions derived from the t-distribution
            Ind = find( Ktable > kurt);
            beta = Beta(Ind(end));
            lambda2 = beta * gamma( (beta-2)/2 )  / ( 2 * M2 * gamma(beta/2) ) ;
            score = (1+beta)*y ./ ( y.^2 + beta/lambda2 );                         % score function
            dscore = -(1+beta)*(y.^2 - beta/lambda2) ./ ( y.^2 + beta/lambda2).^2; % derivative of score function
            ScoreType = 1;

        elseif M4 < 2.6
            % If the desired source signal is strongly sub-Gaussian, then use
            % the score function derived from the generalized Gaussian distribution.
            % But for simplicity, here we use the well-known fixed function: y=x^3
            score =  y.^3;                % score function
            dscore = 3 * y.^2;            % derivative of score function
            ScoreType = 2;
            %             score = y - tanh(y);          % score function
            %             dscore = 1 - sech(y).^2;      % derivative of score function


        else
            % If the desired source signal is near Gaussian or/and skewed, then use
            % the score functions derived from GBD and Pearson system, because
            % ordinary higher-order statistics based ICA algorithm has poor
            % performance.
            if M4 < 1.2 * M3^2 + 2.2
                % When sub-Gaussian distributions are fitted to the nearest GBD
                % GBD sample min & max estimation used.
                % For details, see gbd_momentfit.m, gbd_score.m and Ref.[4]
                samplemin = min(y);
                samplemax = max(y);
                beta = gbd_momentfit(M3,M4,samplemin,samplemax,IClen);
                [score  dscore] = gbd_score(y,beta);
                ScoreType = 3;

            else
                % Use Pearson system
                % Pearson moment estimation used.
                % For details, see Ref.[3]
                C = 10*M4 - 12*M3^2 - 18;
                b0 = -( 4*M4 - 3*M3^2 ) / C;
                b1 = -M3 * (M4 +3) / C;
                b2 = -(2*M4 - 3*M3^2 - 6) / C;
                denominator = b0 + b1*y + b2*y.^2;
                score = -(y-b1)./denominator;                                   % score function
                dscore = -(b0 + b1^2 + 2*b1*b2*y - b2*y.^2)./(denominator.^2);  % derivative of score function
                ScoreType = 4;

            end   % end of selecting GBD or Pearson System model

        end   % end of selection of score functions

        % Use the Newton-like optimal algorithm to update the weight vector
        w = mean(dscore) * w - mu * ( X * score' )/IClen;
        w = w/norm(w);

        distance = 1-abs(w'*w1);   % distance between the current w and w1
        if distance < bound
            % If the current w is near to w1, then judge whether the
            % algorithm has converged.
            wchange = 1-abs(w'*oldw);
            if trace==1   fprintf('No.%d iteration: change in w is %g\n',loop, wchange);  end;
            if wchange < epsilon
                flag = 0;
                if trace==1
                    fprintf('Converged after %d iteration\n',loop);
                    if ScoreType == 1
                        fprintf('The t-distribution for super-Gaussian was used\n');
                    elseif ScoreType == 2
                        fprintf('The generalized Gaussian distribution for sub-Gaussian was used\n');
                    elseif ScoreType == 3
                        fprintf('The generalized Beta distribution was used\n');
                    elseif ScoreType == 4
                        fprintf('The Pearson system was used\n');
                    end
                end;
            end

        else
            % If the current w is not near to w1, then change current w to w1,
            % added to a small random deviation. This step is important
            % to guarantee that the solution is spatially near w1
            if trace==1
                fprintf('After No.%d iteration, the distance between current weight and w1 is %g \n',loop,distance);
                fprintf('  Clearly, the current weight is far from the one derived from the first stage, \n');
                fprintf('  therefore change the current weight to w1, added a small random deviation\n');
            end
            w = w1 + 0.05 * randn(size(w));
            w = w/norm(w);
        end

        if loop >= maxIter
            if trace==1
                fprintf('\n------------------------------------ Notice ------------------------------------\n');
                fprintf('After %d iterations, the algorithm still cannot converge, implying that the weight\n',loop);
                fprintf('vector w1 from the first stage may be more reliable than the one from the second stage.\n');
                fprintf('Thus we use w1 as the final weigtht vector !\n');
                fprintf('--------------------------------------------------------------------------------\n');
            end;
            w = w1;
            flag = 0;
        end

        oldw = w;
        loop = loop + 1;

    end  % end of WHILE loop
end  % end of "elseif type == 2"


% the final output
y = w'*X;

if trace==1        
    fprintf('\n=================================================================\n');
    fprintf('                     End of TCExt Algorithm \n');
    fprintf('=================================================================\n');
end





function beta=gbd_momentfit(alpha3,alpha4,samplemin,samplemax,samplen)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GBD_MOMENTFIT - Estimates the parameters of the Generalized Beta
% Distribution (GBD) using the first four sample moments
% (the method of moments). Alternatively, the sample minimum and maximum
% can be used instead of the sample mean and variance.
%
% Input
%   alpha3    = sample 3rd moment
%   alpha4    = sample 4th moment
%   samplemin = sample mean     or sample minimum
%   samplemax = sample variance or sample maximum
%   samplelen =                    sample length
%
% Output
%   beta = the distribution parameters vector [beta(1) beta(2) beta(3) beta(4)]
%
% Copyright (c) Helsinki University of Technology,
% Signal Processing Laboratory,
% Jan Eriksson, Juha Karvanen, and Visa Koivunen.
%
% Last modified: 5.9.2000
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nargin<4,  disp(['Usege: gbd_momentfit(alpha3,alpha4,samplemin,', ...
        'samplemax,samplen) or']);
    disp(['gbd_momentfit(alpha3,', ...
        'alpha4,alpha1,alpha2)']);
    error('Not enough input arguments.');
end

if or(alpha4<=1+alpha3^2,alpha4>=3+2*alpha3^2),
    warning(['The moment values are not covered by the '...
        'GBD distribution.']);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Moment estimates for beta(3) and beta(4). These are obtained from
% the equations (15) and (16) in
%   Eriksson, J., Karvanen, J., and Koivunen, V.:
%   "Source Distribution Adaptive Maximum Likelihood Estimation of
%   ICA Model", Proceedings of Second International Workshop on
%   Independent Component Analysis and Blind Signal Separation,
%   Helsinki 2000, pp. 227--232
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
s=alpha3^2;
k=alpha4;
a=-((3*(-1 + k - s)*(k^2*(-32 + s) + ...
    k*(96 + 78*s - sqrt(s)*sqrt(k^2*(-32 + s) - ...
    9*s*(7 + 4*s) + 6*k*(16 + 13*s))) - ...
    3*(21*s + 12*s^2 + sqrt(s)*sqrt(k^2*(-32 + s) - ...
    9*s*(7 + 4*s) + 6*k*(16 + 13*s)))))/ ...
    ((2*k - 3*(2 + s))*(k^2*(-32 + s) - 9*s*(7 + 4*s) + ...
    6*k*(16 + 13*s))));

b=-(3*(-1 + k - s)*(k^2*(-32 + s) - 63*s - 36*s^2 + ...
    3*sqrt(s)*sqrt(k^2*(-32 + s) - 9*s*(7 + 4*s) + ...
    6*k*(16 + 13*s)) + ...
    k*(96 + 78*s + sqrt(s)*sqrt(k^2*(-32 + s) - ...
    9*s*(7 + 4*s) + 6*k*(16 + 13*s)))))/ ...
    (2*k^3*(-32 + s) + k^2*(384 + 246*s - 3*s^2) + ...
    27*s*(14 + 15*s + 4*s^2) - 18*k*(32 + 49*s + 17*s^2));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Note: A=beta(3)+1, B=beta(4)+1 correspond to the Statistics Toolbox
% values used for the ordinary Beta Distribution
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if alpha3>0,
    beta(3)=a-1;
    beta(4)=b-1;
else
    beta(3)=b-1;
    beta(4)=a-1;
end;

if nargin==5
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Estimates for beta(1) and beta(2) based on sample minimum and
    % maximum. The PDF of any GBD distribution is nonzero on interval [beta(1),
    % beta(1)+beta(2)], therefore this "ad-hoc" estimation guarantees
    % that the corresponding pdf is non-zero for all realizations
    % (this is not the case with the moment estimation, which may
    % lead to problems when calculating the score function...).
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    beta(1)=(samplen+1)*samplemin/samplen;
    beta(2)=(samplen+1)*samplemax/samplen-beta(1);
else
    alpha1=samplemin;
    alpha2=samplemax;
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Moment estimates for beta(1) and beta(2). These are given by
    % equations (17) and (18) in
    %   Eriksson, J., Karvanen, J., and Koivunen, V.:
    %   "Source Distribution Adaptive Maximum Likelihood Estimation of
    %   ICA Model", Proceedings of Second International Workshop on
    %   Independent Component Analysis and Blind Signal Separation,
    %   Helsinki 2000, pp. 227--232
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    beta(2)=(beta(3)+beta(4)+2)*...
        sqrt(alpha2*(beta(3)+beta(4)+3)/((beta(3)+1)*(beta(4)+1)));
    beta(1)=alpha1-beta(2)*(beta(3)+1)/(beta(3)+beta(4)+2);
end

% The end of the function %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%





function [phi,phi2]=gbd_score(x,beta)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GBD_SCORE -  Calculates the score function and its derivative
% for a distribution from the Generalized Beta Distribution.
%
% Input:
%   x    = the realization matrix (sample values)
%   beta = the distribution parameters vector [beta(1) beta(2) beta(3) beta(4)]
%
% Output:
%   phi   = the score function matrix corresponding to x
%   phi2  = the derivative of the score function matrix corresponding to x
%
% Copyright (c) Helsinki University of Technology,
% Signal Processing Laboratory,
% Jan Eriksson, Juha Karvanen, and Visa Koivunen.
%
%
% Last modified: 5.9.2000
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nargin<2, disp('Usege: gbd_score(x,beta)');
    error('Not enough input arguments.');
end

delta=0.01;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The values of x are forced to interval [beta(1)+delta,beta(1)+beta(2)-delta]
% to ensure the numerical stability.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
x=max((beta(1)+delta),min(x,beta(1)+beta(2)-delta));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The functions are given by the equations (22) and (23) in
%   Eriksson, J., Karvanen, J., and Koivunen, V.:
%   "Source Distribution Adaptive Maximum Likelihood Estimation of ICA Model",
%   Proceedings of Second International Workshop on
%   Independent Component Analysis and Blind Signal Separation,
%   Helsinki 2000, pp. 227--232
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
phi =(beta(2)*beta(3)-x.*(beta(3)+beta(4))+beta(1)*...
    (beta(3)+beta(4)))./((x-beta(1)).*(-x+beta(1)+beta(2)));
phi2=-(beta(3)+beta(4))./((x-beta(1)).*(-x+beta(1)+beta(2)))...
    +phi./(-x+beta(1)+beta(2))-phi./(x-beta(1));

% The end of the function %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


