function [y,w] = akExtr( X, w, tmdelay, lamda, OverValue, maxIter,type)
% akExtr--signal extraction algorithm based on joint maximization of autocorrelation and kurtosis.
%            It extracts one source signal, exploiting its 
%            special autocorrelation information. So, before running this
%            algorithm, one should know the desired source(S_i)'s
%            autocorrelation information, i.e., the optimal time delay t^*:
%                  E{ S_i * S_i(t-t^*) }  not.equ. 0
%                  E{ S_j * S_j(t-t^*) }  equ. 0 .
%            It returns the desired source y and corresponding weight
%            vector w(y=w'*X). Notice the input observed mixed signals X
%            should be prewhitened.
%
% Command:
%    [y,w] = akExtr( X, w, tmdelay, lamda, OverValue,maxIter,type)
%
% Parameters:
%          y --- extracted source signal, according to the given optimal
%                time delay.
%          w --- corresponding weight vector, that is, y=w'*X.
%          X --- prewhitened observed mixed signals. Each row is a mixed
%                signal.
%    tmdelay --- optimal time delay, i.e., the autocorrelation value at
%                time delay tmdelay is far from zero, while the other
%                sources' autocorrelation value at the time delay is nearly
%                zero.
%      lamda --- a positive constant with large value. lamda = 50-200 is good for
%                successful extraction of the desired signal.
%  OverValue --- stopping criterion, say, 0.0001. When the changed value 
%                of weighted vector is less than it, then the algorithm 
%                stops.
%    maxIter --- maximum iterations for estimating each independent
%                component.
%       type --- 1-algorithm based on standard kurtosis and autocorrelation
%                2-algorithm based on generalized kurtosis and
%                autocorrelation, which is more robust to outliers
%
% Reference:
%    [1] Zhi-Lin Zhang, Zhang Yi: Extraction of temporally correlated
%    sources with its application to non-invasive fetal electrocardiogram
%    extraction. Neurocomputing 69 (7-9) (2006) 900-904
%    [2] A.Hyvarinen, E.Oja: A fast fixed-point algorithm for independent
%    component analysis. Neural Computation 9:1483-1492,1997
%
% Author: Zhilin Zhang
%
% version: 1.1     Date: Aug.17,2005    % update the computation of autocorrelation matrix
% version: 1.0     Date: Apr.21,2005
% 


fprintf('Starting akExtr algorithm ..\n');
[ICnum, IClen]=size(X);

oldw = w;

flag = 1;
loop = 1;

if type==1
    while (flag == 1)
        
        % compute the autocorrelation matrix   
        Rx = X(:, tmdelay+1:IClen) * X(:, 1: IClen-tmdelay)' / (IClen-tmdelay); 
        
        % a part of the algorithm
        tem =  X * ( X'* w).^3 / IClen  -  3 * w ;
        
        % updating rule
        % Rx + Rx' can be approximated as 2*Rx, but lost robustness to estimate
        % error of time delay
        w = tem +  lamda /(loop+1)* (Rx+Rx')*w;

        % normalization
        w = w/norm(w); 
        
        % decide whether the algorithm has converged
        wchange = 1-abs(w'*oldw);
%         fprintf('No.%d iteration: change in w is %g\n',loop, wchange);
        if wchange < OverValue 
            fprintf('Converged after %d iteration\n',loop);
            flag = 0;
        end
        
        
        if loop >= maxIter
            fprintf('After %d iteration, still cannot convergent.\n',loop);
            flag = 0;
        end
        
        oldw = w;
        loop = loop + 1;
        
    end
    
elseif type==2
    while (flag == 1)
        
        % compute the autocorrelation matrix
        Rx = X(:, tmdelay+1:IClen) * X(:, 1: IClen-tmdelay)' / (IClen-tmdelay); 
        
        y = w' * X;
        Ey3x = X * (y.^3)'/IClen;
        Ey4  = mean(y.^4);
        tem2 = Ey3x/Ey4 - w;
        
        % updating rule
        % Rx + Rx' can be approximated as 2*Rx, but lost robustness to estimate
        % error of time delay
        w = tem2 +  lamda /(loop+1)* (Rx+Rx')*w;
        
        w = w/norm(w);
        
        % decide whether the algorithm has converged
        wchange = 1-abs(w'*oldw);
%         fprintf('No.%d iteration: change in w is %g\n',loop, wchange);
        if wchange < OverValue 
            fprintf('Converged after %d iteration\n',loop);
            flag = 0;
        end
        
        if loop >= maxIter
            fprintf('After %d iteration, still cannot convergent.\n',loop);
            flag = 0;
        end
        
        oldw = w;
        W(:,loop)=w;
        loop = loop + 1;
        
    end
else
    fprintf('You should choose a valid type:\n');
    fprintf('  1 -- algorithm based on standard kurtosis and autocorrelation \n');
    fprintf('  2 -- algorithm based on generalized kurtosis and autocorrelation\n');
    return;
end


% output
y = w'* X;

fprintf('End of akExtr algorithm !\n');