function [y,w] = eigBSE( X, tmdelay, index, w, trace)
% eigBSE--Temporally correlated signal extraction based on eigenvalue decomposition
%        It extracts a source signal, exploiting its 
%        special autocorrelation information. So, before running this
%        algorithm, one should know the desired source(S_i)'s
%        autocorrelation information, i.e., the optimal time delay t^*:
%              E{ S_i * S_i(t-t^*) }  not.equ. 0
%              E{ S_j * S_j(t-t^*) }  equ. 0 .
%        or, strictly but practically:
%              E{ S_i * S_i(t-t^*) } > E{ S_j * S_j(t-t^*) }.
%        It returns the desired source y and corresponding weight vector w(y=w'*X). 
%        If there are several source signals having the same period length T, then
%        the algorithm still extracts these signals one by one, providing that they
%        have different autocorrelation values at lag T.
%        Note the input observed mixed signals X should be prewhitened.
%
% Command:
%    [y,w] = eigBSE( X, tmdelay, index, w, trace)
%    [y,w] = eigBSE( X, tmdelay, index, w)
%    [y,w] = eigBSE( X, tmdelay, index)
%    [y,w] = eigBSE( X, tmdelay);
%
% Parameters:
%          y --- extracted source signal, according to the given optimal
%                time delay.
%          w --- corresponding weight vector, that is, y=w'*X.
%          X --- prewhitened observed mixed signals. Each row is a mixed
%                signal.
%    tmdelay --- a vector. Its each element represents an optimal time delay, i.e., 
%                the autocorrelation value at the time delay tmdelay is far from zero, 
%                while the other sources' autocorrelation value at the time delay is nearly
%                zero.
%      index --- if there are several source signals having the same period length T,
%                then INDEX indicates which source signal is extracted that
%                has the INDEX largest autocorrelation value at the lag T.
%      trace --- print out the progress
%
% See also:
%    eigBSEdemo       eigBSEonline
%
% Reference:
%    [1] Zhi-Lin Zhang, Zhang Yi, Robust  Extraction of Specific Signals with 
%        Temporal Structure, Neurocomputing 69 (7-9) 888-893 
%
% Author: Zhilin Zhang
%         z4zhang@ucsd.edu
%
% version: 1.2.1     Date: Apr.26, 2006
% version: 1.1       Date: Apr. 3, 2006
% version: 1.0       Date: July 25, 2005
%
%    


[ICnum, IClen]=size(X);

if nargin == 4
    trace = 1;
elseif nargin == 3
    trace = 1; w = randn(ICnum,1); w = w/norm(w);
elseif nargin == 2
    trace = 1; w = randn(ICnum,1); w = w/norm(w); index = 1;
elseif nargin == 1
    error('The input parameter number cannot be only ONE');
end

if trace==1   fprintf('\nStarting eigBSE algorithm ..\n');  end


%------------------------------------------------------------------------------------
% compute the multiple delayed covariance matrix of observed data
Rx = zeros(size(X,1));
for k = tmdelay
    tempo = X(:,k+1:IClen) * X(:,1: IClen-k)' / (IClen-k);
    Rx = Rx + tempo;
end
clear tempo;
Rx = (Rx + Rx')/2;
[V,D] = eig(Rx);   
[puiss,II] = sort( diag(D) );

eigs = flipud(diag(D));
numSource = find( eigs> -eigs(1) * 0.1 );

if trace==1  
    fprintf('The first largest %d eigenvalues are :\n',numSource(end));
    disp(eigs(1:numSource(end)));
    fprintf('Note: There may be %d source signals with the same autocorrelation value\n',numSource(end));   
    fprintf('      And you are extracting the No.%d source signal.\n',index);
end

w = V(:,II(ICnum-index+1));  % select the eigenvector corresponding to the INDEX largest eigenvalue

y = w'* X;

if trace==1  fprintf('End of eigBSE algorithm !\n\n');   end

%------------------------------------------------------------------------------------
